/*
 * This file is part of SionTower.
 *
 * 
 * David Saltares Marquez (C) 2011
 * <david.saltares@gmail.com>
 *
 * 
 * SionTower examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * SionTower examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SionTower examples.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#ifndef SIONTOWER_TRUNK_SRC_INCLUDE_COLLISIONMANAGER_H_
#define SIONTOWER_TRUNK_SRC_INCLUDE_COLLISIONMANAGER_H_

#include <OGRE/Ogre.h>
#include <list>
#include <boost/unordered_map.hpp>
#include <boost/function.hpp>

#include "body.h"
#include "shape.h"

//! Gestor que registra los bodies, detecta colisions y proporciona un sistema de callbacks

/**
 *  @author David Saltares M&aacute;rquez
 *  @date 02-02-2011
 *
 *  CollisionManager lleva el registro de los elementos colisionables (Body)
 *  que componen la escena. Realiza tests de colisi&oacute;n entre dichos bodies y
 *  llama al callback oportuno si es que lo hubiese. Sigue el patr&oacute;n de dise&ntilde;o
 *  Singleton, es decir, una &uacute;nica instancia accesible desde todo el sistema.
 *  No es un sistema de f&iacute;sicas, s&oacute;lo detecta colisiones e informa de ellas.
 *
 *  Para que un cuerpo sea considerado como colisionable debe de ser registrado
 *  en el CollisionManager. Los cuerpos tienen un tipo determinado (entero)
 *  &uacute;til para clasificarlos dentro de la gesti&oacute;n de colisiones. Debemos
 *  indicarle expl&iacute;citamente al sistema que detecte e informe de las colisiones
 *  entre dos tipos de cuerpos. Esto lo conseguimos a trav&eacute;s del sistema de
 *  callbacks. Un callback es una funci&oacute;n (independiente o miembro de alguna
 *  clase) que tiene la siguiente forma:
 *
 *  <code>
 *  void collisionCallbacK(Body* bodyA, Body* bodyB);
 *  </code>
 *
 *  Para registrar un callback utilizamos boost::bind y boost:funcion y le
 *  indicamos al CollisionManager qu&eacute; tipos de cuerpos estar&aacute;n asociados. En
 *  cada iteraci&oacute;n del bucle de juego debemos llamar al m&eacute;todo
 *  CollisionManager::checkCollisions para detectar las colisiones y llamar a
 *  los callbacks. A continuaci&oacute;n de muestra de un ejemplo de uso.
 *
 *  \code
 *  // Al comienzo de la aplicaci&oacute;n creamos el CollisionManager
 *  CollisionManager* collisionManager = new CollisionManager();
 *  ...
 *  
 *  // Tenemos bodyA, bodyB y bodyC, los registramos en el CollisionManager
 *  CollisionManager::getSingleton().addBody(bodyA);
 *  CollisionManager::getSingleton().addBody(bodyB);
 *  CollisionManager::getSingleton().addBody(bodyC);
 *
 *  // Registramos un callback para la colisi&oacute;n entre bodies de tipo 1 y 2
 *  boost::function<bool(Body*, Body*)> callback = boost::bind(&miFuncionCallback, _1, _2);
 *  CollisionManager::getSingleton().addCollisionCallback(callback, 1, 2);
 *
 *  ...
 *  
 *  // En el bucle de juego
 *  while(!exit) {
 *      // Actualizar elementos
 *
 *      // Comprobar colisiones
 *      CollisionManager::getSingleton().checkCollisions();
 *
 *      // Renderizar
 *  }
 *  
 *  ...
 *
 *  // Al cerrar la aplicaci&oacute;n, destruimos el CollisionManager
 *  delete collisionManager;
 *  \endcode
 *
 */
class CollisionManager: public Ogre::Singleton<CollisionManager> {
    public:
        /**
         *  Objecto funci&oacute;n que modela los callback de colisiones: recibe dos
         *  cuerpos y no devuelve nada
         */
        typedef boost::function<void(Body*, Body*)> CollisionCallback;
       
        /**
         *  Constructor
         *
         *  Configura el reparto de colisiones de la clase Shape (ya no es
         *  necesario llamar a Shape::configureCollisionDispatching de forma
         *  manual).
         */
        CollisionManager();

        /**
         *  Destructor
         *
         *  No libera la memoria ocupada por los cuerpos registrados, debes
         *  destruirlos de forma manual. Si el cuerpo est&aacute; bajo el control
         *  de la clase GameObject o derivados no ser&aacute; necesario, ella lo
         *  har&aacute; de forma autom&aacute;tica.
         */
        ~CollisionManager();

        /**
         *  @return &uacute;nica instancia de la clase CollisionManager
         */
        static CollisionManager& getSingleton();

        /**
         *  @return puntero a la &uacute;nica instancia de la clase CollisionManager
         */
        static CollisionManager* getSingletonPtr();

        /**
         *  @param body cuerpo a a&ntilde;adir
         *
         *  Registra un cuerpo en el gestor de colisiones. A partir de ahora
         *  ser&aacute; tenido en cuenta en los tests de colisi&oacute;n siempre y cuando
         *  haya un callback definido.
         */
        void addBody(Body* body);

        /**
         *  @param body cuerpo a borrar del gestor de colisiones.
         *  @return true si se ha borrado, false si no estaba registrado.
         *
         *  Elimina el registro del body en el gestor de colisiones. No elimina
         *  el cuerpo, si no estaba bajo la gesti&oacute;n de un GameObject debes
         *  hacerlo de forma manual.
         */
        bool removeBody(Body* body);

        /**
         *  Elimina todos los cuerpos bajo el conrol del gestor de colisiones.
         *  No los destruye, si no estaban bajo el control de objetos
         *  GameObject debes hacerlo de forma manual.
         */
        void removeAllBodies();

        /**
         *  @param typeA tipo del primer objeto
         *  @param typeB tipo del segundo objeto
         *  @param c objeto funci&oacute;n que recibe dos punteros a cuerpos y
         *  devuelve un booleano.
         *
         *  Registra un callback para las colisiones entre objetos de tipo
         *  typeA-typeB y viceversa. Cuando se detecte una colisi&oacute;n entre esos
         *  tipos se llamar&aacute; al m&eacute;todo encapsulado por c. Si ya exist&iacute;a un
         *  callback registrado para esos tipos, se sobreescribe.
         */
        void addCollisionCallback(int typeA, int typeB, CollisionCallback c);

        /**
         *  @param typeA tipo del primer objeto
         *  @param typeB tipo del segundo objeto
         *  @return true si se ha eliminado el callback, false si no exist&iacute;a
         *
         *  Elimina el callback entre objetos de tipos typeA-typeB (y
         *  viceversa). Ya no se detectar&aacute;n colisiones entre dichos tipos.
         */
        bool removeCollisionCallback(int typeA, int typeB);

        /**
         *  Elimina todos los callbacks registrados en el gestor de colisiones.
         */
        void removeAllCollisionCallbacks();

        /**
         *  Efect&uacute;a todos los tests de colisi&oacute;n oportunos entre los cuerpos
         *  registrados y llama a los callbacks pertinentes.
         */
        void checkCollisions();
    private:
        typedef boost::unordered_map<int, boost::unordered_map<int, CollisionCallback> > CollisionCallbackTable;

        CollisionCallbackTable _collisionCallbackTable;
        std::list<Body*> _bodies;
        Sphere* _sphereA;
        Sphere* _sphereB;

        bool existsCallback(int typeA, int typeB, CollisionCallback* collisionCallback);
};

#endif   // SIONTOWER_TRUNK_SRC_INCLUDE_COLLISIONMANAGER_H_


/**
 *  @mainpage Sistema de colisiones de Sion Tower
 *
 *  <strong><center><h2>David Saltares M&aacute;rquez</h2></center></strong>
 *  <ul>
 *      <li><strong>Correo electr&oacute;nico</strong>: <a href="mailto:david.saltares@gmail.com">david.saltares@gmail.com</a></li>
 *      <li><strong>Web del proyecto</strong>: <a href="http:://siondream.com/blog/category/proyectos/pfc">http:://siondream.com/blog/category/proyectos/pfc</a></li>
 *      <li><strong>Forja de Red Iris</strong>: <a href="http://forja.rediris.es/projects/cusl5-iberogre">http://forja.rediris.es/projects/cusl5-iberogre</a></li>
 *  </ul>
 *
 *  <h3>Descripci&oacute;n</h3>
 *  
 *  <p>Sion Tower es un videojuego libre de estrategia y acci&oacute;n desarrollado
 *  como Proyecto Fin de Carrera por David Saltares M&aacute;rquez. Est&aacute; muy
 *  relacionado con <a href="http://osl2.uca.es/iberogre">IberOgre</a>,
 *  la wiki de Ogre3D en espa&ntilde;ol (del mismo autor).</p>
 *
 *  <p>El sistema de colecciones de Sion Tower puede utilizarse de forma muy
 *  sencilla en otros proyectos que empleen Ogre3D como engine de renderizado.
 *  Est&aacute; liberado bajo licencia GPL v3, consulta el fichero LICENSE.txt distribuido
 *  con el proyecto. Cuenta con caracter&iacute;sticas sencillas pero que cubren las
 *  necesidades de la mayor&iacute;a de proyectos y, en cualquier caso, extender el
 *  sistema no es complicado. Utiliza C++ est&aacute;ndar y es compatible con todas las
 *  plataformas que soporten Boost y Ogre3D.</p>
 *
 *  <ul>
 *      <li>Clase Body modela el aspecto colisionable de los elementos del
 *      juego</li>
 *      <li>Los cuerpos est&aacute;n formados por varios objetos de la clase Shape</li>
 *      <li>Shape es una clase abstracta y cuenta con las siguientes
 *      especializaciones: Sphere, AxisAlignedBox y Plane. A&ntilde;adir nuevas
 *      especializaciones es muy sencillo.</li>
 *      <li>Tests de colisi&oacute;n entre todas las combinaciones de formas
 *      disponibles.</li>
 *      <li>CollisionManager que registra los cuerpos involucrados en posibles
 *      colisiones dentro de la escena.</li>
 *      <li>Filtrado de colisiones seg&uacute;n el tipo de los Body.</li>
 *      <li>Callbacks para las colisiones entre dos tipos concretos de Body.</li>
 *      <li>Particionado del espacio b&aacute;sico para aumentar el rendimiento.</li>
 *      <li>La clase GameObject encapsula la parte colisionable (Body) y visual
 *      (Ogre::Entity / Ogre::SceneNode) de los elementos del juego. Puede ser
 *      especializada de forma sencilla aunque tambi&eacute;n puede utilizarse sin
 *      cambios adicionales.</li>
 *  </ul>
 *
 *  <h3>Dependencias y uso</h3>
 *
 *  <p>El sistema de colisiones de Sion Tower depende de las bibliotecas
 *  <strong>Ogre3D</strong> y <strong>Boost</strong>. En funci&oacute;n de tu sistema
 *  deber&aacute;s instalarlas de manera diferente. Para incorporar este sistema de
 *  colisiones a tu proyecto s&oacute;lo debes a&ntilde;adir su c&oacute;digo fuente. Por supuesto,
 *  debes asegurarte que tu sistema de compilaci&oacute;n tenga en cuenta y enlace
 *  dicho c&oacute;digo (configura tu makefile o proyecto de IDE).</p>
 */
